rescue_configure() {
	local hostname="${1}"
	local release="bookworm"
	local package
	# apt / conf
	printf "\
Acquire::AllowInsecureRepositories False;
Acquire::AllowWeakRepositories False;
Acquire::AllowDowngradeToInsecureRepositories False;
Acquire::Check-Valid-Until True;
APT::Install-Recommends False;
APT::Install-Suggests False;
APT::Get::Show-Versions True;
Dir::Etc::SourceParts \"\";
Dpkg::Progress True;
" >"/etc/apt/apt.conf.d/apt.conf"
	# apt / sources
	printf "%s" "\
deb https://deb.debian.org/debian \
${release} main non-free-firmware contrib non-free
deb https://deb.debian.org/debian \
${release}-backports main non-free-firmware contrib non-free
deb https://deb.debian.org/debian \
${release}-updates main non-free-firmware contrib non-free
deb https://deb.debian.org/debian-security \
${release}-security main non-free-firmware contrib non-free
" >"/etc/apt/sources.list"
	# bash / rc
	main_link_bashrc
	mv .bashrc .bashrc.old
	# host name
	hostname "${hostname}"
	# locales
	printf "\
en_US.UTF-8 UTF-8
fr_FR.UTF-8 UTF-8
" >"/etc/locale.gen"
	# generate locales
	locale-gen
	# update catalog
	apt_update
	# disable frontend
	debian_disable_frontend
	# install backports
	set "tmux"
	for package in "${@}"; do
		echo
		echo "${package}"
		apt-get install --assume-yes \
			--target-release "${release}-backports" \
			"${package}"
		apt_clean
	done
	# install packages
	set "apt-file" "mosh" "screen" "byobu"
	for package in "${@}"; do
		echo
		echo "${package}"
		apt-get install --assume-yes \
			"${package}"
		apt_clean
	done
	# update catalog
	apt_update
}

rescue_install() {
	local package
	local release="bookworm"
	# update catalog
	apt_update
	# disable frontend
	debian_disable_frontend
	# upgrade packages
	apt_upgrade
	# clean cache
	apt_clean
	# install packages
	set \
		"man-db" \
		"dmidecode" "efibootmgr" "lshw" "pciutils" "usbutils" \
		"parted" "mdadm" "cryptsetup-bin" "lvm2" \
		"btrfs-progs" "dosfstools" "duperemove" "squashfs-tools" \
		"git" "micro" "nano" "python3" "rsync" "vim" \
		"exa" "lf" "ncdu" "nnn" "ranger" "tree" \
		"file" "htop" "iotop" "ipcalc" "libdigest-sha3-perl" "lsof"
	for package in "${@}"; do
		echo
		echo "${package}"
		apt-get install --assume-yes \
			"${package}"
		apt_clean
	done
	# install backports
	set \
		"grub-pc-bin" \
		\
		"grub-efi-amd64-bin"
	for package in "${@}"; do
		echo
		echo "${package}"
		apt-get install --assume-yes \
			--target-release "${release}-backports" \
			"${package}"
		apt_clean
	done
}

rescue_upload() {
	local host="${1}"
	local hostname="${2}"
	if [ "${hostname}" ]; then
		local user="root"
		#
		local user_host="${user}@${host}"
		# remove fingerprints
		ssh-keygen -R "${host}"
		# copy ssh id
		ssh-copy-id \
			-o "StrictHostKeyChecking=accept-new" \
			"${user_host}"
		# upload root
		rsync --delete --recursive \
			"${MAIN_BASH_ROOT}/" "${user_host}:/etc/bash/"
		# call setup
		# TODO variable
		ssh "${user_host}" -- "\
source \"/etc/bash/main.sh\" ; rescue_configure \"${hostname}\""
		# create session
		ssh "${user_host}" -- byobu new-session -d
		# send keys
		ssh "${user_host}" -- byobu send-keys "rescue_install" "C-m"
		# attach session
		mosh "${user_host}" -- byobu attach-session
	else
		echo "host & hostname"
		return 1
	fi
}

rescue_wipe_1_zero() {
	dd status="progress" \
		if="/dev/zero" of="/dev/mapper/crypt" bs="1G"
}

rescue_wipe_3_close() {
	umount "/media/boot"
	umount "/media/crypt" &&
		cryptsetup luksClose "crypt"
}
